import { Module } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { AuthenticationModule } from '@modules/authentication/authentication.module';
import { EventEmitterModule } from '@nestjs/event-emitter';
import { HealthModule, HealthModuleOptions } from '@mobicoop/health-module';
import { AuthorizationModule } from '@modules/authorization/authorization.module';
import {
  AUTHENTICATION_REPOSITORY,
  USERNAME_REPOSITORY,
} from '@modules/authentication/authentication.di-tokens';
import { MESSAGE_PUBLISHER } from './modules/messager/messager.di-tokens';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { ICheckRepository } from '@mobicoop/health-module/dist/core/domain/types/health.types';
import { MessagerModule } from './modules/messager/messager.module';

@Module({
  imports: [
    ConfigModule.forRoot({ isGlobal: true }),
    EventEmitterModule.forRoot(),
    HealthModule.forRootAsync({
      imports: [AuthenticationModule, MessagerModule],
      inject: [
        AUTHENTICATION_REPOSITORY,
        USERNAME_REPOSITORY,
        MESSAGE_PUBLISHER,
      ],
      useFactory: async (
        authenticationRepository: ICheckRepository,
        usernameRepository: ICheckRepository,
        messagePublisher: MessagePublisherPort,
      ): Promise<HealthModuleOptions> => ({
        serviceName: 'auth',
        criticalLoggingKey: 'logging.auth.health.crit',
        checkRepositories: [authenticationRepository, usernameRepository],
        messagePublisher,
      }),
    }),
    AuthenticationModule,
    AuthorizationModule,
    MessagerModule,
  ],
  controllers: [],
  exports: [AuthenticationModule, MessagerModule],
})
export class AppModule {}
