import { IQueryHandler, QueryHandler } from '@nestjs/cqrs';
import { Inject, UnauthorizedException } from '@nestjs/common';
import { ValidateAuthenticationQuery } from './validate-authentication.query';
import {
  AUTHENTICATION_REPOSITORY,
  USERNAME_REPOSITORY,
} from '@modules/authentication/authentication.di-tokens';
import { AuthenticationRepositoryPort } from '../../ports/authentication.repository.port';
import { UsernameRepositoryPort } from '../../ports/username.repository.port';
import { AuthenticationEntity } from '@modules/authentication/core/domain/authentication.entity';
import { UsernameEntity } from '@modules/authentication/core/domain/username.entity';
import { AggregateID, NotFoundException } from '@mobicoop/ddd-library';

@QueryHandler(ValidateAuthenticationQuery)
export class ValidateAuthenticationQueryHandler implements IQueryHandler {
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly authenticationRepository: AuthenticationRepositoryPort,
    @Inject(USERNAME_REPOSITORY)
    private readonly usernameRepository: UsernameRepositoryPort,
  ) {}

  execute = async (
    query: ValidateAuthenticationQuery,
  ): Promise<AggregateID> => {
    let usernameEntity: UsernameEntity;
    try {
      usernameEntity = await this.usernameRepository.findByName(query.name);
    } catch (e) {
      throw new NotFoundException();
    }
    let authenticationEntity: AuthenticationEntity;
    try {
      authenticationEntity = await this.authenticationRepository.findOneById(
        usernameEntity.getProps().userId,
      );
    } catch (e) {
      throw new NotFoundException();
    }
    try {
      const isAuthenticated = await authenticationEntity.authenticate(
        query.password,
      );
      if (isAuthenticated) return authenticationEntity.id;
      throw new UnauthorizedException();
    } catch (e) {
      throw new UnauthorizedException();
    }
  };
}
