import { AggregateID, AggregateRoot } from '@mobicoop/ddd-library';
import {
  CreateUsernameProps,
  UpdateUsernameProps,
  UsernameProps,
} from './username.types';
import { UsernameAddedDomainEvent } from './events/username-added.domain-event';
import { UsernameDeletedDomainEvent } from './events/username-deleted.domain-event';
import { UsernameUpdatedDomainEvent } from './events/username-updated.domain-event';

export class UsernameEntity extends AggregateRoot<UsernameProps> {
  protected readonly _id: AggregateID;

  static create = async (
    create: CreateUsernameProps,
  ): Promise<UsernameEntity> => {
    const props: UsernameProps = { ...create };
    const username = new UsernameEntity({
      id: props.name,
      props: {
        name: props.name,
        userId: props.userId,
        type: props.type,
      },
    });
    username.addEvent(
      new UsernameAddedDomainEvent({ aggregateId: props.name }),
    );
    return username;
  };

  update(props: UpdateUsernameProps): void {
    this.props.name = props.name;
    this.addEvent(
      new UsernameUpdatedDomainEvent({
        aggregateId: props.name,
      }),
    );
  }

  delete(): void {
    this.addEvent(
      new UsernameDeletedDomainEvent({
        aggregateId: this.id,
      }),
    );
  }

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
