import { Inject, Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import {
  LoggerBase,
  MessagePublisherPort,
  PrismaRepositoryBase,
} from '@mobicoop/ddd-library';
import { AuthenticationEntity } from '../core/domain/authentication.entity';
import { AuthenticationRepositoryPort } from '../core/application/ports/authentication.repository.port';
import { PrismaService } from './prisma.service';
import { AuthenticationMapper } from '../authentication.mapper';
import { AUTH_MESSAGE_PUBLISHER } from '../authentication.di-tokens';

type AuthenticationBaseModel = {
  uuid: string;
  password: string;
};

export type AuthenticationReadModel = AuthenticationBaseModel & {
  usernames: UsernameModel[];
  createdAt: Date;
  updatedAt: Date;
};

export type AuthenticationWriteModel = AuthenticationBaseModel & {
  usernames: {
    create: UsernameModel[];
  };
};

export type UsernameModel = {
  username: string;
  type: string;
};

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class AuthenticationRepository
  extends PrismaRepositoryBase<
    AuthenticationEntity,
    AuthenticationReadModel,
    AuthenticationWriteModel
  >
  implements AuthenticationRepositoryPort
{
  constructor(
    prisma: PrismaService,
    mapper: AuthenticationMapper,
    eventEmitter: EventEmitter2,
    @Inject(AUTH_MESSAGE_PUBLISHER)
    protected readonly messagePublisher: MessagePublisherPort,
  ) {
    super(
      prisma.auth,
      prisma,
      mapper,
      eventEmitter,
      new LoggerBase({
        logger: new Logger(AuthenticationRepository.name),
        domain: 'auth',
        messagePublisher,
      }),
    );
  }
}
