import { UsernameDeletedDomainEvent } from '@modules/authentication/core/domain/events/username-deleted.domain-event';
import { UsernameUpdatedDomainEvent } from '@modules/authentication/core/domain/events/username-updated.domain-event';
import { UsernameEntity } from '@modules/authentication/core/domain/username.entity';
import {
  CreateUsernameProps,
  Type,
} from '@modules/authentication/core/domain/username.types';

const createUsernameProps: CreateUsernameProps = {
  userId: '165192d4-398a-4469-a16b-98c02cc6f531',
  type: Type.EMAIL,
  name: 'john.doe@email.com',
};

describe('Username entity create', () => {
  it('should create a new username entity', async () => {
    const usernameEntity: UsernameEntity = await UsernameEntity.create(
      createUsernameProps,
    );
    expect(usernameEntity.id).toBe('john.doe@email.com');
    expect(usernameEntity.domainEvents.length).toBe(1);
  });
});
describe('Username entity update', () => {
  it('should update a username entity', async () => {
    const usernameEntity: UsernameEntity = await UsernameEntity.create(
      createUsernameProps,
    );
    usernameEntity.update({
      name: 'new-john.doe@email.com',
    });
    expect(usernameEntity.getProps().name).toBe('new-john.doe@email.com');
    expect(usernameEntity.domainEvents.length).toBe(2);
    expect(usernameEntity.domainEvents[1]).toBeInstanceOf(
      UsernameUpdatedDomainEvent,
    );
  });
});
describe('Username entity delete', () => {
  it('should delete a username entity', async () => {
    const usernameEntity: UsernameEntity = await UsernameEntity.create(
      createUsernameProps,
    );
    usernameEntity.delete();
    expect(usernameEntity.domainEvents.length).toBe(2);
    expect(usernameEntity.domainEvents[1]).toBeInstanceOf(
      UsernameDeletedDomainEvent,
    );
  });
});
