import { HttpService } from '@nestjs/axios';
import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { lastValueFrom } from 'rxjs';
import { DecisionMakerPort } from '../core/application/ports/decision-maker.port';
import {
  Action,
  ContextItem,
  Domain,
} from '../core/domain/authorization.types';

@Injectable()
export class OpaDecisionMaker extends DecisionMakerPort {
  constructor(
    private readonly configService: ConfigService,
    private readonly httpService: HttpService,
  ) {
    super();
  }

  decide = async (
    domain: Domain,
    action: Action,
    context: ContextItem[],
  ): Promise<boolean> => {
    const reducedContext = context.reduce(
      (obj, item) => Object.assign(obj, { [item.name]: item.value }),
      {},
    );
    try {
      const { data } = await lastValueFrom(
        this.httpService.post<Decision>(
          `${this.configService.get<string>('OPA_URL')}${domain}/${action}`,
          {
            input: {
              ...reducedContext,
            },
          },
        ),
      );
      return data.result.allow;
    } catch (e) {
      return false;
    }
  };
}

type Decision = {
  decision_id: string;
  result: {
    allow: boolean;
  };
};
